from abc import ABC, abstractmethod

import paramiko
import getpass

class SSH_Manager(ABC):
    def __init__(self, host, username: str, password: str, port: int):
        self.host = host
        self.username = username
        self.password = password
        self.port = port
        
    @abstractmethod
    def connect(self):
        pass
    
    @abstractmethod
    def sendCommand(self, command):
        pass
    
    @abstractmethod
    def set_up_connection(self):
        pass
    
    def __enter__(self):
        return self.set_up_connection()
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        pass
    
class PxSSH(SSH_Manager):        
    def connect(self):
        self.shell.login(self.host, self.username, self.password, port=self.port)

    def sendCommand(self, command):
        self.shell.sendline(command)
        self.shell.prompt()
        return (self.shell.before).decode()
    
    def set_up_connection(self):
        self.shell = pxssh.pxssh()
        self.connect()
        
        return self.shell
    
class SSH_Handler(SSH_Manager):
    def connect(self):
        self.shell.connect(self.host, username=self.username, password=self.password, port=self.port)

    def sendCommand(self, command):
        _stdin, _stdout,_stderr = client.exec_command(command)
        
        return _stdout.read().decode()
    
    def set_up_connection(self):
        self.shell = paramiko.client.SSHClient()
        self.shell.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.connect()
        
        return self.shell
        
if __name__ == "__main__":
    # Required to set up OpenSSH... not this computer
    username = getpass.getpass('username: ')
    password = getpass.getpass('password: ')
    port = int(input('Port: '))

    commands = ['uname', 'ls', 'cd /', 'ls -al']

    def run_commands(shell):
        for command in commands:
            print(shell.sendCommand(shell))
            
    with SSH_Handler('localhost', username, password, 22) as shell:
        run_commands(shell)
